package org.vcs.bazaar.eclipse.core.commands;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.runtime.IProgressMonitor;
import org.vcs.bazaar.client.IBazaarProgressListener;

public class CommandProgressListener implements IBazaarProgressListener {

	protected final IProgressMonitor monitor;
	protected int worked;
	protected int maxdiff;

	public CommandProgressListener(IProgressMonitor monitor) {
		this.monitor = monitor;
	}

	Pattern FETCHING_REVISIONS_PATTERN = Pattern.compile(".*?Fetching revisions:Inserting stream:Estimate.*?(\\d+)/(\\d+)"); 

	public void logProgress(String message) {
		if (monitor != null) {
			if (message != null) {
				Matcher m = FETCHING_REVISIONS_PATTERN.matcher(message);
				if (m.matches()) {
					int newWorked = 0;
					if (worked == 0) {
						newWorked = 5;
					}
					/* extract total and completed */
					/* bzr seems to increase both total and completed when it encounters merged revisions */
					int completedWork = Integer.parseInt(m.group(1));
					int totalWork = Integer.parseInt(m.group(2));
					int diff = totalWork - completedWork;
					/* calculate new difference between total and completed */
					if (diff > maxdiff) {
						maxdiff = diff;
					} else if (maxdiff > 0) {
						/* it looks like we made some progress */
						int w = 5 + (maxdiff - diff) * 90 / maxdiff;
						if (w > newWorked) {
							newWorked = w;
						}
					}
					if (newWorked > worked) {
						monitor.worked(newWorked - worked);
						worked = newWorked;
					}
				}
			}
			monitor.setTaskName(message);
		}
	}

}
